c notch_filter:  subroutine to conduct notch filtering to seismic data
c to be used with C-code xqrnotch 
c
c Version of notch_filter written by D. Okaya, adapted by R. Abercrombie
c  to allow notch width to be controlled and to filter out harmonics as
c  required.
c  November 1 1993.
c
c
	subroutine notch_filter(nsamples,dt,data,ntfft,coeff,cdata,freq,
     a                        fwidth, nharm, taper, tlen)
	integer*4 data(nsamples),nsamples,ntfft,nharm,tlen
	real	  dt,coeff(ntfft),freq,fwidth
	real      taper(tlen)
	complex   cdata(ntfft)

			call fortmessage('NOTCH_FILTER: ENTERED ROUTINE')
			call fortprint(nsamples,dt,ntfft)

c clear weights to 100%
			call fortmessage('NOTCH_FILTER: clear_coeff routine')
	call clear_coeff(coeff,ntfft)
c call other notches if desired
			call fortmessage('NOTCH_FILTER: set_notch routine')
	call set_notch(coeff,ntfft,dt,freq,fwidth,nharm)


c transfer integer data() to complex cdata()
			call fortmessage('NOTCH_FILTER: set_data routine')
	call set_data(data,nsamples,cdata,ntfft)
c transform and weight
			call fortmessage('NOTCH_FILTER: FFT1D routine')
	call FFT1D(cdata,ntfft, 1.)
			call fortmessage('NOTCH_FILTER: weight routine')
	call weight(coeff,ntfft,cdata)
c retrieve
			call fortmessage('NOTCH_FILTER: FFT1D INV routine')
	call FFT1D(cdata,ntfft, -1.)
			call fortmessage('NOTCH_FILTER: get_data routine')
	call get_data(data,nsamples,cdata,ntfft)


			call fortmessage('NOTCH_FILTER: LEAVING ROUTINE')
	RETURN
	end

	subroutine clear_coeff(coeff,ntfft)
	dimension coeff(ntfft)

	do i=1,ntfft
		coeff(i)=1.0
	enddo
	RETURN
	end



c use a cos*2 taper function.
	subroutine set_notch(coeff,ntfft,dt,fnotch,fwidth,nharm)
c	integer*4 tlen
c	real taper(tlen)
        dimension taper(1000)
	real coeff(ntfft)
	real dt,fnotch,pi,Fnyq,fwidth
	integer jnotch,ntfft,nharm,notchsiz
c
	Fnyq=1/(2.*dt)
	df=Fnyq/float(ntfft/2)
c jnotch = point corresponding to frequency want to notch
	jnotch= ifix(fnotch/df) + 1
c notchsiz = number of points on either side of jnotch to notch
        notchsiz=int(fwidth/df)
c        notchsiz=tlen
c
c cos(1)= cos ( pi/2 / 10)   ~ 1
c cos(10)=cos ( pi/2 )       =0
	pi=3.14159265
	do i=1,notchsiz
		taper(i)=cos(.5*pi*float(i)/notchsiz)
		taper(i)=taper(i)*taper(i)
	enddo

c	write(*,*)'notch: Fnyq, df, jnotch',fnyq,df,jnotch

c Make notch at first frequency
c low side around jnotch, for +/- frequencies
	do i=1,notchsiz
		coeff(jnotch-notchsiz+i)=coeff(jnotch-notchsiz+i)*taper(i)
		coeff(ntfft+2-(jnotch-notchsiz+i))=coeff(jnotch-notchsiz+i)
	enddo
c high side around jnotch, for +/- frequencies
	do i=1,notchsiz
		coeff(jnotch+i-1) =coeff(jnotch+i-1)*taper(notchsiz+1-i)
		coeff(ntfft+2-(jnotch+i-1))=coeff(jnotch+i-1)
	enddo
c
c Apply to harmonics as requested:
        if (nharm.ge.2) then
           jnotch2=jnotch*2
c low side around jnotch2, for +/- frequencies
	do i=1,notchsiz
		coeff(jnotch2-notchsiz+i)=coeff(jnotch2-notchsiz+i)*taper(i)
		coeff(ntfft+2-(jnotch2-notchsiz+i))=coeff(jnotch2-notchsiz+i)
	enddo
c high side around jnotch2, for +/- frequencies
	do i=1,notchsiz
		coeff(jnotch2+i-1) =coeff(jnotch2+i-1)*taper(notchsiz+1-i)
		coeff(ntfft+2-(jnotch2+i-1))=coeff(jnotch2+i-1)
	enddo
        endif
c
        if (nharm.ge.3) then
           jnotch3=jnotch*3
c low side around jnotch3, for +/- frequencies
	do i=1,notchsiz
		coeff(jnotch3-notchsiz+i)=coeff(jnotch3-notchsiz+i)*taper(i)
		coeff(ntfft+2-(jnotch3-notchsiz+i))=coeff(jnotch3-notchsiz+i)
	enddo
c high side around jnotch3, for +/- frequencies
	do i=1,notchsiz
		coeff(jnotch3+i-1) =coeff(jnotch3+i-1)*taper(notchsiz+1-i)
		coeff(ntfft+2-(jnotch3+i-1))=coeff(jnotch3+i-1)
	enddo
        endif
        if (nharm.ge.4) then
           jnotch4=jnotch*4
c low side around jnotch4, for +/- frequencies
	do i=1,notchsiz
		coeff(jnotch4-notchsiz+i)=coeff(jnotch4-notchsiz+i)*taper(i)
		coeff(ntfft+2-(jnotch4-notchsiz+i))=coeff(jnotch4-notchsiz+i)
	enddo
c high side around jnotch4, for +/- frequencies
	do i=1,notchsiz
		coeff(jnotch4+i-1) =coeff(jnotch4+i-1)*taper(notchsiz+1-i)
		coeff(ntfft+2-(jnotch4+i-1))=coeff(jnotch4+i-1)
	enddo
        endif

	RETURN
	end


	subroutine set_data(data,nsamples,cdata,ntfft)
	integer data(nsamples)
	complex cdata(ntfft)

	do i=1,nsamples
		x=float(data(i))
		cdata(i)=CMPLX(x,0.)
	enddo

	do i=nsamples+1,ntfft
		cdata(i)=CMPLX(0.,0.)
	enddo
	RETURN
	end



	subroutine weight(coeff,ntfft,cdata)
	real coeff(ntfft)
	complex cdata(ntfft)

	do i=1,ntfft
		cdata(i)=cdata(i)*coeff(i)
	enddo
	RETURN
	end

	subroutine get_data(data,nsamples,cdata,ntfft)
	integer data(nsamples)
	complex cdata(ntfft)

	do i=1,nsamples
		x=real(cdata(i))
		data(i)= ifix(x)
	enddo
	RETURN
	end
