c notch_filter:  subroutine to conduct notch filtering to seismic data
c to be used with C-code xqrnotch 
c
c
	subroutine notch_filter(nsamples,dt,data,ntfft,coeff,cdata)
	integer*4 data(nsamples),nsamples,ntfft
	real	  dt,coeff(ntfft)
	complex   cdata(ntfft)

			call fortmessage('NOTCH_FILTER: ENTERED ROUTINE')
			call fortprint(nsamples,dt,ntfft)

c clear weights to 100%
			call fortmessage('NOTCH_FILTER: clear_coeff routine')
	call clear_coeff(coeff,ntfft)
c call other notches if desired
			call fortmessage('NOTCH_FILTER: set_notch routine')
	call set_notch(coeff,ntfft,dt,60.)


c transfer integer data() to complex cdata()
			call fortmessage('NOTCH_FILTER: set_data routine')
	call set_data(data,nsamples,cdata,ntfft)
c transform and weight
			call fortmessage('NOTCH_FILTER: FFT1D routine')
	call FFT1D(cdata,ntfft, 1.)
			call fortmessage('NOTCH_FILTER: weight routine')
	call weight(coeff,ntfft,cdata)
c retrieve
			call fortmessage('NOTCH_FILTER: FFT1D INV routine')
	call FFT1D(cdata,ntfft, -1.)
			call fortmessage('NOTCH_FILTER: get_data routine')
	call get_data(data,nsamples,cdata,ntfft)


			call fortmessage('NOTCH_FILTER: LEAVING ROUTINE')
	RETURN
	end

	subroutine clear_coeff(coeff,ntfft)
	dimension coeff(ntfft)

	do i=1,ntfft
		coeff(i)=1.0
	enddo
	RETURN
	end



c use a cos*2 taper function.
	subroutine set_notch(coeff,ntfft,dt,fnotch)
	dimension coeff(ntfft),taper(10)
	real dt,fnotch,pi,Fnyq
	integer jnotch,ntfft

c cos(1)= cos ( pi/2 / 10)   ~ 1
c cos(10)=cos ( pi/2 )       =0
	pi=3.14159265
	do i=1,10
		taper(i)=cos(.5*pi*float(i)/10)
		taper(i)=taper(i)*taper(i)
	enddo

	Fnyq=1/(2.*dt)
	df=Fnyq/float(ntfft/2)
	jnotch= ifix(fnotch/df) + 1

c	write(*,*)'notch: Fnyq, df, jnotch',fnyq,df,jnotch


c low side around jnotch, for +/- frequencies
	do i=1,10
		coeff(jnotch-10+i)=coeff(jnotch-10+i)*taper(i)
		coeff(ntfft+2-(jnotch-10+i))=coeff(jnotch-10+i)
	enddo
c high side around jnotch, for +/- frequencies
	do i=1,10
		coeff(jnotch+i-1) =coeff(jnotch+i-1)*taper(10+1-i)
		coeff(ntfft+2-(jnotch+i-1))=coeff(jnotch+i-1)
	enddo

	RETURN
	end


	subroutine set_data(data,nsamples,cdata,ntfft)
	integer data(nsamples)
	complex cdata(ntfft)

	do i=1,nsamples
		x=float(data(i))
		cdata(i)=CMPLX(x,0.)
	enddo

	do i=nsamples+1,ntfft
		cdata(i)=CMPLX(0.,0.)
	enddo
	RETURN
	end



	subroutine weight(coeff,ntfft,cdata)
	real coeff(ntfft)
	complex cdata(ntfft)

	do i=1,ntfft
		cdata(i)=cdata(i)*coeff(i)
	enddo
	RETURN
	end

	subroutine get_data(data,nsamples,cdata,ntfft)
	integer data(nsamples)
	complex cdata(ntfft)

	do i=1,nsamples
		x=real(cdata(i))
		data(i)= ifix(x)
	enddo
	RETURN
	end
